# -*- coding: utf-8
# Copyright 2001, 2002 by IVA Team and contributors
#
# This file is part of IVA.
#
# IVA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# IVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with IVA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import AccessControl
import OFS, Globals
from OFS import SimpleItem
import types
import time
import re
from TraversableWrapper import Traversable
from Globals import Persistent, PersistentMapping
from AccessControl import ClassSecurityInfo
from AccessControl import allow_module, allow_class, allow_type
from Cruft import Cruft
from common import add_dtml, translate, perm_edit, perm_view

class Kodutoo(
     OFS.SimpleItem.Item,
     Persistent,
     AccessControl.Role.RoleManager,
     Traversable,
     Cruft
     ):

     meta_type = 'Assignment'
     security = ClassSecurityInfo()

     """ Assignment object """
     def __init__(self, REQUEST, tyyp, kirjeldus="", 
         normPunkte=0,kalendriID=None):
       """ init """
       self.tyyp=int(tyyp)
       self.kirjeldus=kirjeldus
       self.loppaeg=None #Esituspiiri pole mratud
       self.normPunkte=normPunkte
       self.kalendriID=None #kalendrisyndmuse id

     def manage_afterAdd(self, item, container):
	 container.grades[self.id] = {}

     def get_id(self):
	 return self.id

     def getDeadline(self, REQUEST):
	 return time.strftime(translate(self,'timestamp_format',target=self.giveLanguage(REQUEST)), time.localtime(self.loppaeg))

     def getGrades(self,user,detail=''):
	""" doc string """
	gradesObj = getattr(self, 'grades')
	if not gradesObj.has_key(self.get_id()):
		gradesObj[self.get_id()] = {}
	grades = gradesObj[self.get_id()]
	if not grades.has_key(user):
		value = float(0)
		self.setUserPoints(self.get_id(), user, detail, value)
	if not gradesObj['Overall'].has_key(user):
		gradesObj['Overall'][user] = {}
	return grades[user][detail]

     def getType(self, REQUEST):
	 """ get assignment type word """
	 return self.getTypes(REQUEST)[self.tyyp]

     def getPoints(self):
	 """ get assignment value in points """
	 return self.normPunkte

     def getTitle(self, REQUEST=None):
       "Antakse kui on"
       if hasattr(self, 'pealkiri'):
         return self.pealkiri
       else:
         return '(untitled)' 

     def getGroups(self):
	 """ gets groupwork groups"""
	 if not hasattr(self, 'grupid'):
		 self.grupid = []
	 return self.grupid
 
     def kodutooMaterjal(self,bs, tulem='',level=1):
	i=1
	aste=""
	if not hasattr(self, 'juhend'):
		self.juhend = ""
	while i!=level:
		aste += "--"
		i=i+1
	for obj in bs.list_contents(criteria='weight'):
		s=obj.absolute_url()[len(self.fle_root().absolute_url()):]
		tulem += "\n<option value=\""+str(s)+"\" "
		if s == self.juhend:
			tulem += "selected=\"selected\""
		tulem +=">"
		tulem += aste+obj.get_name()
		tulem += "</option>"
		if obj.meta_type == 'WebtopFolder' and not obj.kasWikiKaust():
			tulem = self.kodutooMaterjal(obj,tulem,level+1)
	return tulem 

     def changeAssignmentHandler(self, REQUEST):
       "Muutuste salvestus"
       eelmine_loppaeg = self.loppaeg
       eelmine_punktid = self.normPunkte
       self.normPunkte=float(re.sub(',', '.', REQUEST.normPunkte))
       self.kirjeldus=REQUEST.kirjeldus
       self.pealkiri=REQUEST.pealkiri
       if hasattr(REQUEST,'juhend'):
	       if REQUEST.juhend == "Valimata":
		       self.juhend = ""
	       else:
		       self.juhend = REQUEST.juhend
       else:
	       self.juhend = ""
       if self.tyyp==0:
         self.testiID=REQUEST.testiID
       if self.tyyp==2:
         self.grupid=[]
         if hasattr(REQUEST, 'grupid'):
           if type(REQUEST.grupid) is types.StringType:
             self.grupid.append(REQUEST.grupid)
           else:
             for x in REQUEST.grupid:
               self.grupid.append(x)
	 subgfolder = self.aq_parent.subgroups
         for x in self.grupid:
           tmp=getattr(subgfolder, x)  #self.aq_parent.leiaGrupiKaust(x)
	   if not hasattr(tmp.aq_self, 'portfolio'):
		   tmp.aq_self.lisa_kaust('portfolio')
		   folder_name = 'portfolio' #XXX: translate me!
		   tmp.aq_self.portfolio.set_name(folder_name)
		   tmp.aq_self.portfolio.setObjPermission(20) #XXX: me permission code required!
		   tmp.aq_self.portfolio.setSubgroupName(x)
	   grupikaust = getattr(tmp, 'portfolio')
           if not hasattr(grupikaust.aq_self, self.id):
             grupikaust.lisa_kaust(self.id)
             getattr(grupikaust.aq_self, self.id).seaEsitluskaust(1)
             getattr(grupikaust.aq_self, self.id).set_name(self.pealkiri)
	     getattr(grupikaust.aq_self, self.id).setObjPermission(20)
	     getattr(grupikaust.aq_self, self.id).setSubgroupName(x)
       if hasattr(REQUEST, 'kasLoppaeg'):
         import YlTest
         from YlTest import LahendusLuba
         abi=LahendusLuba() #vaid ajamen tarbeks
         self.loppaeg=abi.ajaPyydmisLeht(REQUEST, 'loppaeg')
       else:
         self.loppaeg=None
       self.absolute_url()
       try:
	       if self.grupid == []:
		       REQUEST.RESPONSE.redirect('../manage_assignments')
		       return
       except:
	       pass
       if self.tyyp == 2:
	       target = self.grupid
	       #self.kalendriID = []
       else:
	       target = []
	       target.append('blaah')
       j = -1
       wiga = 1
       uusID = []
       for sisegr in target:
	       j = j+1
	       if self.tyyp == 2:
		       syndmused = getattr(self.find_course().subgroups, sisegr).syndmused
	       else:
		       syndmused = getattr(self.find_course(), 'syndmused')
	       s_nimi = translate(self,"Task",target=self.giveLanguage(REQUEST))
	       s_nimi = str(self.pealkiri)
	       kt_tyyp = self.getTypes(REQUEST)[self.tyyp]
	       s_lyhi = str(self.pealkiri)
	       if self.tyyp != 0 and self.tyyp != 3:
	          s_status = 1
	       else:
	          s_status = 0
	       if self.kalendriID != None and self.kalendriID != []:
		  try:
	              try:
                      	if self.tyyp != 2:
		        	too_syndmus = getattr(syndmused, self.kalendriID)
		      	else:
				too_syndmus = getattr(syndmused, self.kalendriID[j])
		      except:
			      pass
	              s_pikk = too_syndmus.pikk
	              s_end = too_syndmus.lopp
	              s_res = too_syndmus.siseviide
	              if s_end != eelmine_loppaeg and eelmine_loppaeg != None:
 		         s_pikk += "<br />"+ translate(self,'Deadline updated: ',target=self.giveLanguage(REQUEST))
	                 if self.loppaeg !=None:
	                   s_pikk += time.strftime(translate(self,'timestamp_format',target=self.giveLanguage(REQUEST)), time.localtime(self.loppaeg))
	                 else:
	                   s_pikk += translate(self, 'No deadline',target=self.giveLanguage(REQUEST))
	                 s_end = self.loppaeg
		      if eelmine_punktid != self.normPunkte:
			      s_pikk += "<br />Punktid muutunud: "+str(self.normPunkte)
	              syndmused._delObject(too_syndmus.id)
		      wiga = 0
	          except:
			  pass
	       if wiga: 
	          s_end = time.time()
	          if hasattr(REQUEST, 'kasLoppaeg'):
	             s_end = abi.ajaPyydmisLeht(REQUEST, 'loppaeg')
	          s_pikk = 'L_ia_task_added'+" "+self.pealkiri+"("+kt_tyyp+")<br />"+self.kirjeldus+""
		  s_pikk += "<br />"+'L_ia_points'+": "+str(self.normPunkte)
	          if self.tyyp != 0 and self.tyyp != 3:
		     s_pikk += "<br />"+'L_ia_deadline'+": "
	             s_pikk += time.strftime(translate(self,'timestamp_format',target=self.giveLanguage(REQUEST)), time.localtime(s_end))
	       try:
		       s_algus = s_end
	       except:
		       s_algus = time.time()
		       s_end = time.time()
	       too_syndmus = syndmused.lisaSyndmusKohe(s_nimi,s_lyhi,s_pikk,s_algus,s_end,'',self.juhend,s_status)
	       too_syndmus.setIconTag('images/kodune')
	       if self.tyyp == 2:
		       uusID.append(too_syndmus.id)
	       else:
		       self.kalendriID = too_syndmus.id
			
       self.kalendriID = uusID
       REQUEST.RESPONSE.redirect('../manage_assignments')

     def assignmentGrading_handler(self, REQUEST, save, user, points, comment):
	 """ assignmentGrading form handler. Saves user grade """
	 try:
	 	points = float(points)
       	 except:
	 	return self.message_dialog(message='Invalid value for points. Must be numerical!')
	 if self.tyyp==1:
		 self.setUserPoints(self.get_id(), user, 'points', points)
	 	 self.setUserPoints(self.get_id(), user, 'comment', comment)
	 if self.tyyp!=1:
		 for x in self.kasutajad_sisegrupis(user):
			 self.setUserPoints(self.get_id(), x, 'points', points)
			 self.setUserPoints(self.get_id(), x, 'comment', comment)
	 return REQUEST.RESPONSE.redirect(self.absolute_url()+'/manage_gradeAssignment')


class KodutoodeKataloog(
     OFS.Folder.Folder,
     OFS.SimpleItem.Item,
     Persistent,
     AccessControl.Role.RoleManager,
     Traversable,
     Cruft
     ):

     """ Assignments folder """

     meta_type = "Assignments folder"
     security = ClassSecurityInfo()

     # XXX:Migration needed for Persistent mapping
     def __init__(self):
	""" self.grades mapping example:
	    grades = {
		'too1':{
			'Vahur':{
				'points':'55.3',
				'comment':'comment box',
				'teacherComment':'teacher comment box'
				}
			}
		'Overall':{
			'Vahur':{
				'TotPoints':'90.1',
				'TotPointsComment':'Total points comment',
				'grade':'F',
				'gradeCom':'Total grade comment',
				'OverallComment':'Overall comment'
				}
			}
		}
	"""
	self.grades = PersistentMapping()
	self.grades['Overall'] = {}
	self.grades = self.grades
	self.hindeTahised=["A", "B", "C", "D", "E", "F"]
	self.hindePiirid=[90, 80, 70, 60, 50]
	self.tooNr=0
     
     def getTypes(self, REQUEST):
       "Assignment types"
       t = [translate(self, 'Quiz',target=self.giveLanguage(REQUEST)), translate(self, 'Product',target=self.giveLanguage(REQUEST)), translate(self, 'Groupwork',target=self.giveLanguage(REQUEST)), translate(self,'Presentation',target=self.giveLanguage(REQUEST))]
       return t

     def getOverallDetails(self, user, detail=''):
	""" return things """
	gradesObj = getattr(self, 'grades')
	try:
		return gradesObj['Overall'][user][detail]
	except KeyError:
		return

     def index_html(self,REQUEST):
	 """ redirect to manage_assignments """
	 return REQUEST.RESPONSE.redirect('manage_assignments')

     def lisaKodutoo(self, REQUEST, tyyp):
       "Luuakse vastava tbiga kodut ja vljastatakse selle eksemplar"
       self.tooNr=self.tooNr+1
       id='too'+str(self.tooNr)
       too=Kodutoo(REQUEST, tyyp)
       too.id=id
       self._setObject(too.id, too)
       return too
       
     def addAssignment_handler(self, REQUEST):
       "Andmete salvestus"
       too=self.lisaKodutoo(REQUEST, REQUEST.tyyp)
       if too.tyyp==2:
	       too.grupid = []
       REQUEST.RESPONSE.redirect(self.absolute_url()+'/'+too.id+'/manage_changeAssignment')
       
     def manageAssignFormHandler(self, REQUEST, kustuta='', salvesta='', tooID=()):
       "Toiming vastavalt nupule"
       if kustuta:
         if type(tooID)==types.StringType:
           tooID=(tooID,)
         for x in tooID:
           self._delObject(x)
       if salvesta:
         self.hindeTahised=[]
         self.hindePiirid=[]
         kokku=len(REQUEST.hindePiir)
         veel=1
         i=0
         while veel and i<kokku:
           try:
#             print i
             self.hindeTahised.append(REQUEST.hindeTahis[i])
             self.hindePiirid.append(float(re.sub(',', '.', REQUEST.hindePiir[i])))
#             if (i>0) and (self.hindePiirid[i]<=self.hindePiirid[i-1]):
#               veel=0
           except:
             veel=0
           i=i+1
       self.aq_parent.kontrolliHinneteKaustad(REQUEST)             
       REQUEST.RESPONSE.redirect('manage_assignments')
       
     def normpunktideSumma(self, REQUEST=None):
       "Kataloogis olevate tde normpunktide summa"
       summa=0
       for x in self.objectValues():
         summa+=x.normPunkte
       return summa
       
     def punktideleVastavHinne(self, REQUEST, punkte):
       "Arvutatakse this punktide jrgi"
       if not hasattr(self, 'hindePiirid'): return translate(self, 'No scale',target=self.giveLanguage(REQUEST))
       if len(self.hindePiirid)==0: return translate(self, 'No grades', target=self.giveLanguage(REQUEST))
       if self.normpunktideSumma(REQUEST)<=0:
         return ""
       try:
	       protsent=float(punkte)*100/self.normpunktideSumma(REQUEST)
       except:
	       return ""
       i=0
       while i<len(self.hindePiirid):
         if protsent >= self.hindePiirid[i]:
           return self.hindeTahised[i]
         i+=1
       if i<len(self.hindeTahised):
         return self.hindeTahised[i]
       return translate(self, 'Failed', target=self.giveLanguage(REQUEST))

     def listAssignments(self):
	""" return a list of assignments """
	return self.objectValues()

     security.declarePrivate('getUserPoints')
     def setUserPoints(self,assignmentId, user, detail, value):
	if not self.grades.has_key(assignmentId):
		self.grades[assignmentId] = PersistentMapping()
	if not self.grades[assignmentId].has_key(user):
		self.grades[assignmentId][user] = {}
		self.grades[assignmentId][user]['points'] = float(0)
		self.grades[assignmentId][user]['comment'] = ''
		self.grades[assignmentId][user]['teacherComment'] = ''
	self.grades[assignmentId][user][detail] = value
	self.grades = self.grades
	self._p_changed = 1
	self.grades._p_changed = 1

     def setUserOverall(self, user, detail, value):
	self.grades._p_changed = 1
	if not self.grades['Overall'].has_key(user):
		self.grades['Overall'][user] = dict()
		self.grades['Overall'][user]['TotPoints'] = float(0)
		self.grades['Overall'][user]['TotPointsCom'] = ''
		self.grades['Overall'][user]['grade'] = ''
		self.grades['Overall'][user]['gradeCom'] ='' 
		self.grades['Overall'][user]['overallComment'] = ''
	self.grades['Overall'][user][detail] = value
	self.grades = self.grades
	self._p_changed = 1
	self.grades._p_changed = 1

     def manage_gradeAssignment_handler(self, REQUEST, kas_emailida=''):
	""" save grades and comments """
	users = self.get_all_users()
	for y in users:
		for x in self.listAssignments():
			for z in ('points','teacherComment'):
				arg = 'REQUEST.'+x.get_id()+'__'+y.get_id()+'__'+z
				o = eval(arg)
				if z == 'points':
					o = float(re.sub(',','.',str(o)))
					try:
						o = float(o)
					except ValueError:
						continue
				self.setUserPoints(x.get_id(), y.get_id(), z, o)
		for w in ('TotPoints','TotPointsCom','grade','gradeCom','overallComment'):
			arg = 'REQUEST.'+y.get_id()+'__'+w
			o = eval(arg)
			if z == 'TotPoints':
				o = re.sub(',','.',str(o))
				o = float(o)
			self.setUserOverall(y.get_id(), w, o)
	"""
	XXX:add functinality
        if kas_emailida:
          kiri += "\n\n"+REQUEST['L_ia_grademail_body1']
          kiri += "\n\nKursuse "+self.find_course().get_name()
          kiri += REQUEST['L_ia_grademail_body2']
          try:
                kiri += kc.koondhinne + "\n\n"
          except:
                kiri += REQUEST['L_ia_none']+"\n\n"
        if x.get_email().strip() and hasattr(x,'email_hinded') and kas_emailida:
           if x.email_hinded:
                ootavad_kirjad.append((x.get_email().strip(),kiri))
      if not kas_emailida:
        return REQUEST.RESPONSE.redirect('kasutajateHinneteTabel')
      emailsubj = REQUEST['L_ia_grademail_subject']
      for kiri in ootavad_kirjad:
        self.saadaEmail(kiri[0],emailsubj,str(kiri[1]),kiri[0])
      REQUEST.RESPONSE.redirect('kasutajateHinneteTabel')

	"""
	return REQUEST.RESPONSE.redirect(self.absolute_url()+'/manage_gradeAssignment')

class KodutooHinnang(
     OFS.SimpleItem.Item,
     Persistent,
     AccessControl.Role.RoleManager,
     Traversable,
     Cruft
     ):
     "Kodut kommenteeritud andmete hoidla"
     
     def __init__(self, opetajaPunkte, kommentaar):
       "Loomine"
       self.opetajaPunkte=opetajaPunkte
       self.kommentaar=kommentaar
       
     def kysiOpetajaPunkte(self, REQUEST):
       "Vljastus"
       return self.opetajaPunkte
       
     def kysiKommentaar(self, REQUEST):
       "Vljastus"
       return self.kommentaar

Globals.InitializeClass(Kodutoo)
Globals.InitializeClass(KodutoodeKataloog)
Globals.InitializeClass(KodutooHinnang)
       
     
